﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MathGame
{
    class Player
    {
        private Vector2 pos;
        private Vector2 speed;
        private static Animation animNow;
        private static Animation animRight, animLeft, animUp, animDown;

        // player is controled by keys
        // in other case is going straight right (+PlayerSpeed, 0)
        private bool keyCtrl;

        public Player(Vector2 pos, Animation left, Animation right, Animation up, Animation down)
        {
            this.pos = pos;
            keyCtrl = true;

            animDown = down;
            animLeft = left;
            animRight = right;
            animUp = up;

            animNow = animRight;
        }

        public Player(Vector2 pos)
        {
            this.pos = pos;
            this.keyCtrl = true;

            animNow = animRight;
        }

        public bool CheckCollision(Rectangle colRec)
        {
            if (!keyCtrl)
                return false;

            Rectangle plRec = Rectangle();
            if (plRec.Intersects(colRec))
                return true;
            else
                return false;
        }

        public Rectangle Rectangle()
        {
            return new Rectangle((int)pos.X, (int)pos.Y, animNow.GetSprite().Width, animNow.GetSprite().Height);
        }

        public void MoveToNextScreen()
        {
            keyCtrl = false;
        }

        public int Update(int elapsedTime, KeyboardState ks)
        {
            animNow.Update(elapsedTime);
            int baseSpeed = Method.PlayerSpeed;
            if(keyCtrl) {
                if (ks.IsKeyDown(Keys.Left)) {
                    speed.X = -baseSpeed;
                    animNow = animLeft;
                }
                else if (ks.IsKeyDown(Keys.Right)) {
                    speed.X = baseSpeed;
                    animNow = animRight;
                }
                else speed.X = 0;

                if (ks.IsKeyDown(Keys.Down)) {
                    speed.Y = baseSpeed;
                    if (speed.X == 0) animNow = animDown;
                }
                else if (ks.IsKeyDown(Keys.Up)) {
                    speed.Y = -baseSpeed;
                    if (speed.X == 0) animNow = animUp;
                }
                else speed.Y = 0;
            }
            else {
                speed.X = baseSpeed;
                speed.Y = 0;
                animNow = animRight;
            }
            return UpdatePosition();
        }

        private int UpdatePosition()
        {
            // updating position
            pos.X += speed.X;
            pos.Y += speed.Y;

            // checking window bounds
            if (pos.X < 0)
            {
                pos.X = 0;
            }
            if (pos.Y < 0)
            {
                pos.Y = 0;
            }
            if (pos.X > Method.WindowWidth)
            {
                return 1;
                //pos.X = Method.WindowWidth - animNow.GetSprite().Width;
            }
            if (pos.Y + animNow.GetSprite().Height > Method.WindowHeight)
            {
                pos.Y = Method.WindowHeight - animNow.GetSprite().Height;
            }
            return 0;
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            Texture2D tmp = (speed.X == 0 && speed.Y == 0) ? animNow.GetSprite(0) : animNow.GetSprite();
            spriteBatch.Draw(tmp, Rectangle(), Color.White);
        }

    }
}
