﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MathGame
{
    enum eJustification
    {
        J_LEFT,
        J_RIGHT,
        J_CENTER
    }

    class TextRect
    {
        protected eJustification just;
        protected bool initialised;
        protected Rectangle pos;
        protected Texture2D background;
        protected SpriteFont font;
        protected Color bgCol, fgCol;

        private string text;

        public TextRect()
        {
            initialised = false;
        }
        public TextRect(Texture2D background, SpriteFont font)
        {
            initialised = false;
            this.background = background;
            this.font = font;
        }

        public void Init(int x, int y, int w, int h, Color fgCol, Color bgCol, eJustification just)
        {
            initialised = true;
            if(string.IsNullOrEmpty(text)) this.text = "";
            this.fgCol = fgCol;
            this.bgCol = bgCol;
            this.pos = new Rectangle(x, y, w, h);
            this.just = just;
        }

        public void SetText(string text)
        {
            this.text = text;
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            if(!initialised)
                return;

            spriteBatch.Draw(background, pos, bgCol);

            if (string.IsNullOrEmpty(text))
                return;

            Vector2 txtPos = new Vector2();
            if(just == eJustification.J_CENTER)
            {
                txtPos.X = pos.X + pos.Width / 2 - font.MeasureString(text).X / 2;
                txtPos.Y = pos.Y + pos.Height/2 - font.MeasureString(text).Y/2;
            }
            else if(just == eJustification.J_LEFT)
            {
                txtPos.X = pos.X + 1;
                txtPos.Y = pos.Y + pos.Height / 2 - font.MeasureString(text).Y / 2;
            }
            else //if(just == eJustification.J_RIGHT)
            {
                txtPos.X = pos.X + pos.Width - font.MeasureString(text).X;
                txtPos.Y = pos.Y + pos.Height / 2 - font.MeasureString(text).Y / 2;
            }
            spriteBatch.DrawString(font, text, txtPos, fgCol);
        }
    }
}
