﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MathGame
{
    enum eDifficulty
    {
        Diff_Easy,
        Diff_Medium,
        Diff_Hard,
        Diff_Master,

        // no difficulty - value on start
        Diff_None,
    }

    class ExpressionGenerator
    {
        //**********************************************************
        // Parameters
        //**********************************************************
        private Random rnd;
        private eDifficulty difLvl;

        private bool allowDec;
        private int maxChars;
        private int minChars;

        private int minNumber;
        private int maxNumber;

        private List<Char> operators;
        //**********************************************************
        // C-tor
        //**********************************************************
        public ExpressionGenerator()
        {
            this.difLvl = eDifficulty.Diff_None;
            this.operators = new List<char>();
            this.ChangeDifficulty(eDifficulty.Diff_Easy);
            this.SetUsedOperators();
            rnd = new Random();
        }
        //**********************************************************
        // return count of operators used -- need to check if there is at least 1
        public int SetUsedOperators()
        {
            int cnt = 0;
            operators.Clear();
            if((string)ParameterList.Prm("S_MathExp_UsePlus").GetValue() == "Yes")
            {
                operators.Add('+');
                cnt++;
            }
            if ((string)ParameterList.Prm("S_MathExp_UseMinus").GetValue() == "Yes")
            {
                operators.Add('-');
                cnt++;
            }
            if ((string)ParameterList.Prm("S_MathExp_UseMulti").GetValue() == "Yes")
            {
                operators.Add('*');
                cnt++;
            }
            if ((string)ParameterList.Prm("S_MathExp_UseDivision").GetValue() == "Yes")
            {
                operators.Add('/');
                cnt++;
            }
            return cnt;
        }
        //**********************************************************
        public void ChangeDifficulty(eDifficulty difLvl)
        {
            if (this.difLvl == difLvl)
                return;

            // Setting the constants
            this.difLvl = difLvl;
            switch(difLvl)
            {
                case eDifficulty.Diff_Easy:
                    {
                        minChars = 5;
                        maxChars = 15;

                        minNumber = 2;
                        maxNumber = 20;

                        allowDec = false;
                    }
                    break;
                case eDifficulty.Diff_Medium:
                    {
                        minChars = 7;
                        maxChars = 17;

                        minNumber = 10;
                        maxNumber = 30;

                        allowDec = false;
                    }
                    break;
                case eDifficulty.Diff_Hard:
                    {
                        minChars = 9;
                        maxChars = 19;

                        minNumber = 7;
                        maxNumber = 40;

                        allowDec = true;
                    }
                    break;
                case eDifficulty.Diff_Master:
                    {
                        minChars = 11;
                        maxChars = 21;

                        minNumber = 1;
                        maxNumber = 100;

                        allowDec = true;
                    }
                    break;
            }
        }
        public string GenerateExpression(int length, bool allowBrackets)
        {
            // checking bounds
            if (length > maxChars || length < minChars)
                return "";

            int rng;
            string expr = "";

            for (int i = 0; i < length; i++)
            {
                if(i % 2 == 0) {
                    rng = rnd.Next(101);
                    if(rng % 5 == 0 && allowBrackets) { // Add brackets
                        if(length - i < minChars)
                        {
                            expr += this.GenerateNumber().ToString();
                            continue;
                        }

                        int bLength = rnd.Next(minChars, length - i);

                        if (bLength % 2 == 0)
                            bLength -= 1;

                        allowBrackets = false;
                        i += bLength - 1;
                        expr += "(" + GenerateExpression(bLength, false) + ")";
                    }
                    else { // Add random number 
                        expr += this.GenerateNumber().ToString();
                    }
                } else {
                    rng = rnd.Next(101);
                    for(int j = 0; j < operators.Count; j++) {
                        if (rng % operators.Count == j) {
                            expr += operators[j];
                            break;
                        }
                    }
                }
            }

            for (int i = 0; i < operators.Count; i++)
            {
                if(expr.EndsWith(operators[i].ToString()))
                {
                    expr.Remove(expr.Length - 1, 1);
                    break;
                }
            }
            return expr;
        }

        private double GenerateNumber()
        {
            double ret = 0;
            ret = rnd.Next(minNumber, maxNumber);
            if(allowDec)
            {
                if(rnd.Next(101) %2 == 0)
                    ret -= rnd.NextDouble();
                else
                    ret += rnd.NextDouble();
            }
            return ret;
        }
    }
}
