﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System;
using System.Collections.Generic;
using System.IO;

namespace MathGame
{
    // game states
    enum GameState
    {
        // Game menu
        GST_Menu,
        // Running Game
        GST_Game,
        // Short animation after player's death
        GST_GameCollision,
        // Paused Game
        GST_GamePause,
        // Score result screen
        GST_GameOver,
        // Info Screen -- available in menu
        GST_HighScores,
        // Screen with settings of math expresions
        GST_Settings,
        // End Screen -- timer before exiting program
        GST_End
    }
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Game
    {
        Random rnd;
        // Math Parser for calculating expressions and correcting players answers
        MathParser parser;
        // Object for generating mathematical expressions
        ExpressionGenerator expGen;
        // controlls what will be done 
        GameState g_State;

        // helps finding when user pressed button
        bool oldPausePressed;
        bool pausePressed;

        bool oldEscPressed;
        bool escPressed;

        bool oldEnterPressed;
        bool enterPressed;

        // Button objects for menu
        Button nGameBtn, endBtn, settingsBtn, scoresBtn;

        // EnumTextRects for Settings
        List<EnumTextRect> settTRs;

        // TextRects used to display highscores
        List<TextRect> scoreTRs;

        Texture2D lifeSprite;
        Texture2D blockBackground;
        SpriteFont blockFont;
        // blocks with possible results of Math expression
        List<Block> resBlocks;

        List<Texture2D> fogSprites;

        TextRect mathExprTR;
        TextRect scoreTR;
        TextRect livesTR;
        TextRect nameTR;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Player playerChar;

        int endTimerElapsed;

        string mathExpr;  
        SpriteFont exprFont;

        int PosCorrResult;
        double[] results;
        List<double> resCtrl;

        string playerName;
        KeyboardState oldKS;

        Animation playerDeath;
        Animation fireworks;

        // achieved high score
        bool achievedHS;
        SpriteFont cgFont;

        public Game1()
            : base()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            rnd = new Random();

            // Initialization logic here
            ParameterList.InitPrmList();
            Method.InitConstants(this);
            Scores.LoadHighScores();

            // Initialization of Math Parser and Expression Generator
            expGen = new ExpressionGenerator();
            parser = new MathParser('.');

            UpdateGameDifficulty();

            InitPlayerChar();

            Method.LoadBackgroundTextures(this);
            this.IsMouseVisible = true;

            g_State = GameState.GST_GameOver;
            achievedHS = true;

            //g_State = GameState.GST_Menu;
            fogSprites = new List<Texture2D>();

            playerDeath = new Animation();
            fireworks = new Animation();

            //***************************************************************************
            // Initialization of vars for results blocks
            //***************************************************************************
            blockBackground = Content.Load<Texture2D>("GameThumbnail");
            blockFont = Content.Load<SpriteFont>("exprFont");
            resBlocks = new List<Block>();
            //***************************************************************************
            // TextRects Initialization
            //***************************************************************************
            Texture2D backTextRect = Content.Load<Texture2D>("tr_bkground");
            SpriteFont tr_font = Content.Load<SpriteFont>("exprFont");

            mathExprTR = new TextRect(backTextRect, tr_font);
            mathExprTR.Init(0, 0, 300, 20, Color.Black, Color.White, eJustification.J_CENTER);

            scoreTR = new TextRect(backTextRect, tr_font);
            scoreTR.Init(Method.WindowWidth - 100, 0, 100, 20, Color.Black, Color.White, eJustification.J_CENTER);

            nameTR = new TextRect(backTextRect, tr_font);
            nameTR.Init(10, 10, 300, 20, Color.Black, Color.White, eJustification.J_CENTER);

            //***************************************************************************
            // Score TextRects Initialization
            //***************************************************************************
            scoreTRs = new List<TextRect>();
            TextRect tr;

            for(int i = 0; i < 10; i++)
            {
                tr = new TextRect(backTextRect, tr_font);
                tr.Init(Method.WindowWidth / 2 - 150, 50 + (i * 30), 300, 20, Color.Black, Color.White, eJustification.J_CENTER);
                scoreTRs.Add(tr);
            }

            //***************************************************************************
            // Settings Elements Initialization
            //***************************************************************************
            settTRs = new List<EnumTextRect>();
            EnumTextRect tmp;

            tmp = new EnumTextRect(backTextRect, tr_font, ParameterList.Prm("S_Difficulty"));
            tmp.Init(30, 10, 300, 20, Color.Black, Color.White, eJustification.J_CENTER);
            settTRs.Add(tmp);

            tmp = new EnumTextRect(backTextRect, tr_font, ParameterList.Prm("S_MathExp_UsePlus"));
            tmp.Init(30, 50, 300, 20, Color.Black, Color.White, eJustification.J_CENTER);
            settTRs.Add(tmp);

            tmp = new EnumTextRect(backTextRect, tr_font, ParameterList.Prm("S_MathExp_UseMinus"));
            tmp.Init(30, 90, 300, 20, Color.Black, Color.White, eJustification.J_CENTER);
            settTRs.Add(tmp);

            tmp = new EnumTextRect(backTextRect, tr_font, ParameterList.Prm("S_MathExp_UseMulti"));
            tmp.Init(30, 130, 300, 20, Color.Black, Color.White, eJustification.J_CENTER);
            settTRs.Add(tmp);

            tmp = new EnumTextRect(backTextRect, tr_font, ParameterList.Prm("S_MathExp_UseDivision"));
            tmp.Init(30, 170, 300, 20, Color.Black, Color.White, eJustification.J_CENTER);
            settTRs.Add(tmp);
            //***************************************************************************
            // Buttons Initialization
            //***************************************************************************
            Texture2D btn = Content.Load<Texture2D>("ngame");
            Texture2D btn_sel = Content.Load<Texture2D>("ngame_sel");
            nGameBtn = new Button(new Vector2(Window.ClientBounds.Width / 2, 100), btn, btn_sel);

            btn = Content.Load<Texture2D>("scores");
            btn_sel = Content.Load<Texture2D>("scores_sel");
            scoresBtn = new Button(new Vector2(Window.ClientBounds.Width / 2, 200), btn, btn_sel);

            btn = Content.Load<Texture2D>("settings");
            btn_sel = Content.Load<Texture2D>("settings_sel");
            settingsBtn = new Button(new Vector2(Window.ClientBounds.Width / 2, 300), btn, btn_sel);

            btn = Content.Load<Texture2D>("exit");
            btn_sel = Content.Load<Texture2D>("exit_sel");
            endBtn = new Button(new Vector2(Window.ClientBounds.Width / 2, 400), btn, btn_sel);
            //***************************************************************************
            // Timers Initialization
            //***************************************************************************
            endTimerElapsed = 0;
            //***************************************************************************
            base.Initialize();
            
        }

        // Initialization of new game
        public void InitNewGame()
        {
            Method.ResetGameConstants();
            Method.ResetLimitTimer();

            InitPlayerChar();
            GenerateExpression();
            mathExprTR.SetText(mathExpr);
            scoreTR.SetText("Score: " + Method.Score);
        }

        // Initialization of next screen (next expression to solve)
        public void InitNextScreen()
        {
            InitPlayerChar();
            Method.ResetLimitTimer();
            GenerateExpression();
            mathExprTR.SetText(mathExpr);
        }

        // Reset player position and limit timer
        public void InitNextTry()
        {
            InitPlayerChar();
            Method.ResetLimitTimer();
        }

        public void UpdateGameDifficulty()
        {
            eDifficulty diff = (eDifficulty)ParameterList.Prm("S_Difficulty").value.i;
            expGen.ChangeDifficulty(diff);
            Method.UpdateGameDifficulty(diff);
        }

        // Initialization of player character object
        public void InitPlayerChar()
        {
            if(playerChar == null)
            {
                Animation left = new Animation();
                left.AddSlide(new AnimSlide(Content.Load<Texture2D>("left1"), 50));
                left.AddSlide(new AnimSlide(Content.Load<Texture2D>("left2"), 100));
                left.AddSlide(new AnimSlide(Content.Load<Texture2D>("left3"), 150));

                Animation right = new Animation();
                right.AddSlide(new AnimSlide(Content.Load<Texture2D>("right1"), 50));
                right.AddSlide(new AnimSlide(Content.Load<Texture2D>("right2"), 100));
                right.AddSlide(new AnimSlide(Content.Load<Texture2D>("right3"), 150));

                Animation up = new Animation();
                up.AddSlide(new AnimSlide(Content.Load<Texture2D>("up1"), 50));
                up.AddSlide(new AnimSlide(Content.Load<Texture2D>("up2"), 100));
                up.AddSlide(new AnimSlide(Content.Load<Texture2D>("up3"), 150));

                Animation down = new Animation();
                down.AddSlide(new AnimSlide(Content.Load<Texture2D>("down1"), 50));
                down.AddSlide(new AnimSlide(Content.Load<Texture2D>("down2"), 100));
                down.AddSlide(new AnimSlide(Content.Load<Texture2D>("down3"), 150));

                playerChar = new Player(new Vector2(50, Method.WindowHeight/2), left, right, up, down);
            }
            else
            {
                playerChar = new Player(new Vector2(0, Method.WindowHeight/2));
            }
        }

        public void InitScoreTextRects()
        {
            for(int i = 0; i < scoreTRs.Count; i++)
            {
                tScore sc = Scores.GetScore(i);
                if(string.IsNullOrEmpty(sc.Name))
                {
                    scoreTRs[i].SetText(string.Empty);
                }
                else
                {
                    scoreTRs[i].SetText(sc.ToString());
                }
            }
        }

        private void GenerateExpression()
        {
            if (expGen == null)
                expGen = new ExpressionGenerator();
            if (parser == null)
                parser = new MathParser('.');

            int len;
            do 
            {
                len = rnd.Next(Method.ExprMinLen, Method.ExprMaxLen);
            } 
            while (len % 2 == 0);
            mathExpr = expGen.GenerateExpression(len, true);

            results = new double[Method.AnswersCount];
            resCtrl = new List<double>();

            PosCorrResult = rnd.Next(Method.AnswersCount);
            double corrRes = parser.Parse(mathExpr, false);

            resBlocks.Clear();
            int bH = Method.WindowHeight / Method.AnswersCount;
            for (int i = 0; i < Method.AnswersCount; i++)
            {
                if (i == PosCorrResult)
                    results[i] = corrRes;
                else
                    results[i] = corrRes + GenerateFakeResult();

                Block newBlock = new Block(Method.BlocksPosX, 0 + (i * bH), 50, bH, 
                    blockBackground, blockFont, results[i]);
                resBlocks.Add(newBlock);
            }
        }

        private double GenerateFakeResult()
        {
            double ret = 0;
            while (ret == 0)
            {
                ret = rnd.Next(-5, 5);
            }
            for (int i = 0; i < resCtrl.Count; i++)
            {
                if(ret == resCtrl[i])
                {
                    ret = GenerateFakeResult();
                    break;
                }
            }
            resCtrl.Add(ret);
            return ret;
        }
        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            fogSprites.Add(Content.Load<Texture2D>("fog1"));
            fogSprites.Add(Content.Load<Texture2D>("fog2"));
            fogSprites.Add(Content.Load<Texture2D>("fog3"));
            fogSprites.Add(Content.Load<Texture2D>("fog4"));

            lifeSprite = Content.Load<Texture2D>("life");

            exprFont = Content.Load<SpriteFont>("exprFont");
            cgFont = Content.Load<SpriteFont>("gameOverFont");

            playerDeath.AddSlide(new AnimSlide(Content.Load<Texture2D>("expl1"), 50));
            playerDeath.AddSlide(new AnimSlide(Content.Load<Texture2D>("expl2"), 100));
            playerDeath.AddSlide(new AnimSlide(Content.Load<Texture2D>("expl3"), 150));
            playerDeath.AddSlide(new AnimSlide(Content.Load<Texture2D>("expl4"), 200));
            playerDeath.AddSlide(new AnimSlide(Content.Load<Texture2D>("expl5"), 250));
            playerDeath.AddSlide(new AnimSlide(Content.Load<Texture2D>("expl6"), 300));
            playerDeath.AddSlide(new AnimSlide(Content.Load<Texture2D>("expl7"), 350));

            fireworks.AddSlide(new AnimSlide(Content.Load<Texture2D>("fire1"), 100));
            fireworks.AddSlide(new AnimSlide(Content.Load<Texture2D>("fire2"), 200));
            fireworks.AddSlide(new AnimSlide(Content.Load<Texture2D>("fire3"), 300));
            fireworks.AddSlide(new AnimSlide(Content.Load<Texture2D>("fire4"), 400));
            fireworks.AddSlide(new AnimSlide(Content.Load<Texture2D>("fire5"), 500));
            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            int elapsedTime = (int)gameTime.ElapsedGameTime.Milliseconds;
            MouseState ms = Mouse.GetState();
            KeyboardState ks = Keyboard.GetState();

            switch(g_State)
            {
                    // Logic for Ending screen
                case GameState.GST_End:
                    #region Exit Timer
                    if ((endTimerElapsed += elapsedTime) >= Method.EndTimer)
                    {
                        endTimerElapsed = 0;
                        this.Exit();
                    }
                    #endregion
                    break;

                    // Logic for death animation
                case GameState.GST_GameCollision:
                    playerDeath.Update(elapsedTime);
                    #region Animation over Timer
                    if ((endTimerElapsed += elapsedTime) >= Method.DeathTimer)
                    {
                        endTimerElapsed = 0;
                        g_State = GameState.GST_Game;
                        if (Method.DecreaseLives())
                            InitNextScreen();
                        else
                            GameOver();
                    }
                    #endregion
                    break;

                    // Logic for running game
                case GameState.GST_Game:
                    #region Pause Check
                    pausePressed = Keyboard.GetState().IsKeyDown(Keys.P);
                    if((pausePressed && !oldPausePressed))
                    {
                        g_State = GameState.GST_GamePause;
                    }
                    oldPausePressed = pausePressed;
                    #endregion
                    #region ESC check
                    escPressed = Keyboard.GetState().IsKeyDown(Keys.Escape);
                    if((escPressed && !oldEscPressed))
                    {
                        g_State = GameState.GST_Menu;
                    }
                    oldEscPressed = escPressed;
                    #endregion

                    Method.UpdateLimitTimer(elapsedTime);

                    if(playerChar.Update(elapsedTime, ks) == 1)
                    {
                        InitNextScreen();
                        break;
                    }

                    // check wether player collide with limiting rectangle
                    if(playerChar.CheckCollision(Method.LimitElementRect()))
                    {
                        g_State = GameState.GST_GameCollision;
                        playerDeath.Reset();
                        endTimerElapsed = 0;
                    }

                    // Check wether player collide with any of blocks
                    foreach(Block b in resBlocks) {
                        if(playerChar.CheckCollision(b.GetBlockRect()))
                        {
                            if(b.Value == results[PosCorrResult]) {
                                playerChar.MoveToNextScreen();
                                Method.IncreaseScore();
                                scoreTR.SetText("Score: " + Method.Score);
                            }
                            else {
                                g_State = GameState.GST_GameCollision;
                                playerDeath.Reset();
                                endTimerElapsed = 0;
                            }
                        }
                    }
                    break;

                    // Logic for paused game
                case GameState.GST_GamePause:
                    #region Un-Pause Check
                    pausePressed = Keyboard.GetState().IsKeyDown(Keys.P);
                    if((pausePressed && !oldPausePressed))
                    {
                        g_State = GameState.GST_Game;
                    }
                    oldPausePressed = pausePressed;
                    #endregion
                    #region ESC check
                    escPressed = Keyboard.GetState().IsKeyDown(Keys.Escape);
                    if((escPressed && !oldEscPressed))
                    {
                        g_State = GameState.GST_Menu;
                    }
                    oldEscPressed = escPressed;
                    #endregion
                    break;

                case GameState.GST_GameOver:
                    foreach(Keys k in oldKS.GetPressedKeys())
                    {
                        if (ks.IsKeyUp(k))
                        {
                            AddKeyToPlayerName(k);
                        }
                    }
                    oldKS = ks;

                    fireworks.Update(elapsedTime);

                    #region Enter check
                    enterPressed = Keyboard.GetState().IsKeyDown(Keys.Enter);
                    if((enterPressed && !oldEnterPressed))
                    {
                        g_State = GameState.GST_Menu;
                        if(achievedHS)
                        {
                            tScore tmp;
                            tmp.Name = playerName;
                            tmp.Value = Method.Score;
                            Scores.AddScore(tmp);
                        }
                    }
                    oldEnterPressed = enterPressed;
                    #endregion
                    break;

                    // Logic for settings
                case GameState.GST_Settings:
                    foreach (EnumTextRect etr in settTRs)
                        etr.Update(ms, elapsedTime);

                    #region ESC check
                    escPressed = Keyboard.GetState().IsKeyDown(Keys.Escape);
                    if((escPressed && !oldEscPressed))
                    {
                        g_State = GameState.GST_Menu;
                        ParameterList.SavePrms();
                        UpdateGameDifficulty();
                        Method.SettingsFault = (expGen.SetUsedOperators() == 0);
                    }
                    oldEscPressed = escPressed;
                    #endregion
                    break;

                    // Logic for info
                case GameState.GST_HighScores:

                    #region ESC check
                    escPressed = Keyboard.GetState().IsKeyDown(Keys.Escape);
                    if ((escPressed && !oldEscPressed))
                    {
                        g_State = GameState.GST_Menu;
                    }
                    oldEscPressed = escPressed;
                    #endregion
                    break;

                    // Logic for menu
                case GameState.GST_Menu:
                    Vector2 mousePos = new Vector2(Mouse.GetState().X, Mouse.GetState().Y);
                    bool mousePressed = (Mouse.GetState().LeftButton == ButtonState.Pressed);
                    if (nGameBtn.MouseOver(mousePos) && mousePressed)
                    {
                        if (!Method.SettingsFault)
                        {
                            g_State = GameState.GST_Game;
                            InitNewGame();
                        }
                    }
                    if (scoresBtn.MouseOver(mousePos) && mousePressed)
                    {
                        g_State = GameState.GST_HighScores;
                        InitScoreTextRects();
                    }
                    if (settingsBtn.MouseOver(mousePos) && mousePressed)
                        g_State = GameState.GST_Settings;
                    if (endBtn.MouseOver(mousePos) && mousePressed)
                        g_State = GameState.GST_End;
                    break;
            }

            base.Update(gameTime);
        }

        private void GameOver()
        {
            tScore tmp;
            tmp.Name = "";
            tmp.Value = Method.Score;
            achievedHS = Scores.IsHighscore(tmp);

            playerName = "";
            g_State = GameState.GST_GameOver;
        }

        private void AddKeyToPlayerName(Keys k)
        {
            if (k == Keys.Back && playerName.Length > 0)
                playerName = playerName.Substring(0, playerName.Length - 1);
            else if (k == Keys.Space)
                playerName += " ";
            else if ((k >= Keys.A && k <= Keys.Z))
            {
                if (oldKS.IsKeyDown(Keys.LeftShift) || oldKS.IsKeyDown(Keys.RightShift))
                    playerName += k.ToString().ToUpper();
                else
                    playerName += k.ToString().ToLower();
            }
            else if (k >= Keys.NumPad0 && k <= Keys.NumPad9)
                playerName += k.ToString().Substring(k.ToString().Length - 1);

            nameTR.SetText(playerName);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            if(g_State == GameState.GST_GameOver)
                GraphicsDevice.Clear(Color.DarkGray);
            else 
                GraphicsDevice.Clear(Color.LightSkyBlue);

            // drawing code here
            spriteBatch.Begin();

            // Draw background for actual game state
            spriteBatch.Draw(Method.GetBackground(g_State), new Rectangle(0, 0, Window.ClientBounds.Width,
                Window.ClientBounds.Height), Color.White);

            switch (g_State)
            {
                // 
                case GameState.GST_GameOver:
                    nameTR.Draw(spriteBatch);
                    scoreTR.Draw(spriteBatch);

                    if (achievedHS)
                    {
                        // TODO draw something more to show that highscore was achieved
                        Texture2D tmp = fireworks.GetSprite();
                        spriteBatch.Draw(tmp, new Rectangle(100, 50, tmp.Width, tmp.Height), Color.White);
                        spriteBatch.DrawString(cgFont, "Congratulations! You made it to the Top 10!", new Vector2(30, 180), Color.Black);
                    }
                    break;

                // Drawing while running game
                case GameState.GST_Game:
                case GameState.GST_GamePause:
                case GameState.GST_GameCollision:

                    foreach (Block b in resBlocks)
                        b.Draw(spriteBatch);

                    // draw player character
                    playerChar.Draw(spriteBatch);

                    if(g_State == GameState.GST_GameCollision)
                    {
                        spriteBatch.Draw(playerDeath.GetSprite(), playerChar.Rectangle(), Color.White);
                    }

                    // draw a fog for effect
                    if(Method.IsPlayerFollowed())
                    {
                        spriteBatch.Draw(fogSprites[3], Method.LimitElementRect(),
                            new Color(255, 69, 0, MathHelper.Clamp(0, 0, 500)));
                    }

                    for (int i = 0; i < Method.Lives; i++ )
                    {
                        Rectangle posRect = new Rectangle(0 + (i*30), Method.WindowHeight - 30, 30, 30);
                        spriteBatch.Draw(lifeSprite, posRect, Color.White);
                    }

                    // draw actual math expression
                    mathExprTR.Draw(spriteBatch);
                    scoreTR.Draw(spriteBatch);
                    break;


                // drawing menu
                case GameState.GST_Menu:
                    nGameBtn.Draw(spriteBatch);
                    scoresBtn.Draw(spriteBatch);
                    settingsBtn.Draw(spriteBatch);
                    endBtn.Draw(spriteBatch);
                    break;

                // drawing settings objects
                case GameState.GST_Settings:
                    foreach (EnumTextRect etr in settTRs)
                        etr.Draw(spriteBatch);
                    break;

                // drawing highscores
                case GameState.GST_HighScores:
                    foreach (TextRect tr in scoreTRs)
                        tr.Draw(spriteBatch);
                    break;
            }

            base.Draw(gameTime);

            spriteBatch.End();
        }
    }
}
