﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MathGame
{
    // all parameters like speed of the character, etc. will be stored here
    // to be able change game logic like timers duration etc from one place
    static class Method
    {
        // Width and Height of game window
        private static int winH, winW;
        // X position of blocks
        private static int blocksPosX;

        // Textures for background
        private static Texture2D settingsBkground;
        private static Texture2D menuBkground;
        private static Texture2D gameBkground;
        private static Texture2D endBkground;
        private static Texture2D gameOverBkground;

        // range of possible math expression lengths
        private static int m_maxLen, m_minLen;

        // base speed of player 
        private static int p_speed;
        // score - current 
        private static int p_score;
        // score per Question
        private static int scorePerQ;

        // maximum number of lives
        private static int max_lives;
        // number of lives that remains
        private static int p_lives;

        // duration of end screen in ms
        private static int g_endTimer;
        // duration of death animation
        private static int g_deathTimer;

        // to prevent starting game with bad settings
        private static bool settingsFault;

        // number of possible answers
        private static int answersCount;

        // time player is given to solve 1 math expression (once timerBeforeExp finishes)
        private static int g_timeForExp;
        // time before player is chased by deadly element
        private static int g_timeBeforeExp;
        // time passed since generating expression
        private static int g_expTimerPassed;

        public static int Lives
        {
            get { return Method.p_lives; }
        }
        public static int ExprMaxLen
        {
            get { return Method.m_maxLen; }
        }
        public static int ExprMinLen
        {
            get { return Method.m_minLen; }
        }
        public static int DeathTimer
        {
            get { return Method.g_deathTimer; }
        }
        public static int BlocksPosX
        {
            get { return Method.blocksPosX; }
        }
        public static int AnswersCount
        {
            get { return Method.answersCount; }
        }
        public static bool SettingsFault
        {
            get { return Method.settingsFault; }
            set { Method.settingsFault = value; }
        }
        public static int PlayerSpeed
        {
            get { return Method.p_speed; }
        }
        public static int Score
        {
            get { return Method.p_score; }
        }
        public static int WindowHeight
        {
            get { return Method.winH; }
        }
        public static int WindowWidth
        {
            get { return Method.winW; }
        }
        public static int EndTimer
        {
            get { return Method.g_endTimer; }
        }


        public static void InitConstants(Game1 game)
        {
            max_lives = 3;
            p_speed = 2;
            p_score = 0;
            scorePerQ = 10;

            // Timers
            g_endTimer = 1000;
            g_deathTimer = 1200;
            answersCount = 5;

            winH = game.Window.ClientBounds.Height;
            winW = game.Window.ClientBounds.Width;

            blocksPosX = winW / 2 + 100;
        }

        // set game parameters which differs for each difficulty lvl
        public static void UpdateGameDifficulty(eDifficulty diff)
        {
            switch(diff)
            {
                case eDifficulty.Diff_Easy:
                    g_timeForExp = 15000;
                    g_timeBeforeExp = 15000;
                    m_minLen = 5;
                    m_maxLen = 10;
                    break;

                case eDifficulty.Diff_Medium:
                    g_timeForExp = 10000;
                    g_timeBeforeExp = 10000;
                    m_minLen = 7;
                    m_maxLen = 15;
                    break;

                case eDifficulty.Diff_Hard:
                    g_timeForExp = 5000;
                    g_timeBeforeExp = 5000;
                    m_minLen = 9;
                    m_maxLen = 19;
                    break;

                case eDifficulty.Diff_Master:
                    g_timeForExp = 3000;
                    g_timeBeforeExp = 3000;    
                    m_minLen = 11;
                    m_maxLen = 21;
                    break;
            }

        }

        // Called at each start of new Game
        public static void ResetGameConstants()
        {
            p_score = 0;
            p_lives = max_lives;
        }

        public static void IncreaseScore()
        {
            p_score += scorePerQ;
        }
        public static bool DecreaseLives()
        {
            p_lives--;
            return (p_lives > 0);
        }

        public static void UpdateLimitTimer(int elTime)
        {
            g_expTimerPassed += elTime;
        }
        public static bool IsPlayerFollowed()
        {
            return (g_expTimerPassed > g_timeBeforeExp);
        }
        public static void ResetLimitTimer()
        {
            g_expTimerPassed = 0;
        }
        public static Rectangle LimitElementRect()
        {
            Rectangle ret = new Rectangle();
            ret.X = 0;
            ret.Y = 0;
            if (g_expTimerPassed > g_timeBeforeExp)
            {
                double tmp = ((double)(g_expTimerPassed - g_timeBeforeExp) / g_timeForExp) * blocksPosX;
                ret.Width = (int)tmp;
            }
            else
                ret.Width = 0;
            ret.Height = winH;
            return ret;
        }

        public static void LoadBackgroundTextures(Game1 game)
        {
            gameBkground = game.Content.Load<Texture2D>("game_background");
            settingsBkground = game.Content.Load<Texture2D>("grass");
            menuBkground = game.Content.Load<Texture2D>("grass");
            endBkground = game.Content.Load<Texture2D>("menu_bk");
            gameOverBkground = game.Content.Load<Texture2D>("grass");
        }
        // Getters
        public static Texture2D GetBackground(GameState gs)
        {
            switch(gs)
            {
                case GameState.GST_Menu:
                    return menuBkground;

                case GameState.GST_End:
                    return endBkground;

                case GameState.GST_Game:
                case GameState.GST_GamePause:
                case GameState.GST_GameCollision:
                    return gameBkground;

                case GameState.GST_HighScores:
                case GameState.GST_Settings:
                    return settingsBkground;

                case GameState.GST_GameOver:
                    return gameOverBkground;

                default:
                    return null;
            }
        }
    }
}
