﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace MathGame
{
    class ParameterList
    {
        private const string prmFile = "settings.dat";
        private static List<UsrPrm> list;

        public static void AddStringPrm(string ID, string hint, string def = "")
        {
            foreach (UsrPrm prm in list)
            {
                if(prm.ID == ID)
                {
                    prm.hint = hint;
                    prm.value.s = def;
                    return;
                }
            }

            UsrPrm sPrm = new UsrPrm(PrmType.String, ID, hint, def);
            list.Add(sPrm);
        }

        public static void AddIntPrm(string ID, string hint, int units, int val, int max, int min)
        {
            foreach (UsrPrm prm in list)
            {
                if (prm.ID == ID)
                {
                    prm.hint = hint;
                    prm.value.i = val;
                    prm.max.i = max;
                    prm.min.i = min;
                    prm.units = units;
                    return;
                }
            }

            UsrPrm sPrm = new UsrPrm(PrmType.Int, ID, hint, val, max, min);
            sPrm.units = units;
            list.Add(sPrm);
        }

        public static void AddDoublePrm(string ID, string hint, int units, double val, double max, double min, int prec)
        {
            foreach (UsrPrm prm in list)
            {
                if (prm.ID == ID)
                {
                    prm.hint = hint;
                    prm.value.d = val;
                    prm.max.d = max;
                    prm.min.d = min;
                    prm.prec = prec;
                    prm.units = units;
                    return;
                }
            }

            UsrPrm sPrm = new UsrPrm(PrmType.Double, ID, hint, val, max, min, prec);
            sPrm.units = units;
            list.Add(sPrm);
        }

        public static void AddEnumPrm(string ID, string hint, int def, int max, string[] txtlist)
        {
            foreach (UsrPrm prm in list)
            {
                if (prm.ID == ID)
                {
                    prm.hint = hint;
                    prm.value.i = def;
                    prm.max.i = max;
                    prm.txtList = txtlist;
                    return;
                }
            }

            UsrPrm sPrm = new UsrPrm(PrmType.Enum, ID, hint, def, max, txtlist);
            list.Add(sPrm);
        }

        public static UsrPrm Prm(string id)
        {
            foreach (UsrPrm prm in list)
            {
                if (prm.ID == id)
                    return prm;
            }
            return null;
        }
        public static UsrPrm PrmPos(int i)
        {
            if (0 <= i && i < list.Count)
                return list[i];
            else return null;
        }

        public static int GetPrmCount()
        {
            return list.Count;
        }

        public static void InitPrmList()
        {
            string[] yesNo = { "No", "Yes" };
            string[] diff = { "Beginner", "Rookie", "Advanced", "Expert" };
            list = new List<UsrPrm>();

            AddEnumPrm("S_MathExp_UsePlus", "Include adding", 1, 1, yesNo);
            AddEnumPrm("S_MathExp_UseMinus", "Include subtracting", 1, 1, yesNo);
            AddEnumPrm("S_MathExp_UseMulti", "Include multiplication", 0, 1, yesNo);
            AddEnumPrm("S_MathExp_UseDivision", "Include division", 0, 1, yesNo);

            AddEnumPrm("S_Difficulty", "Difficulty", 0, 3, diff);

            LoadPrms();
        }

        public static void LoadPrms()
        {           
            if(File.Exists(prmFile))
            {
                string[] lines = File.ReadAllLines(prmFile);

                foreach(string s in lines)
                {
                    int val_end = -1;
                    for(int i = 0; i < s.Length; i++)
                    {
                        if (s[i] == '=')
                        {
                            val_end = i - 1;
                            break;
                        }
                    }

                    if (val_end == -1) continue;

                    UsrPrm prm = Prm(s.Substring(0, val_end));

                    if(prm != null)
                    {
                        string tmp = s.Substring(val_end + 3);
                        object val = prm.ParseValue(tmp);
                        prm.SetValue(val);
                    }
                }
            }
        }

        public static void SavePrms()
        {
            StreamWriter sw = File.CreateText(prmFile);

            foreach(UsrPrm prm in list)
            {
                sw.WriteLine(prm.ToString());
            }

            sw.Close();
        }
    }
}
