﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace MathGame
{
    struct tScore
    {
        public int Value;
        public string Name;

        public override string ToString()
        {
            return Name + " - " + Value.ToString();
        }
    }

    static class Scores
    {
        private const string scoreFile = "highscores.dat";
        static private List<tScore> scores;

        static public bool IsHighscore(tScore sc)
        {
            if (scores.Count < 10)
                return true;

            foreach(tScore _sc in scores)
            {
                if (sc.Value > _sc.Value)
                    return true;
            }
            return false;
        }

        static public void AddScore(tScore sc)
        {
            bool added = false;
            for (int i = 0; i < scores.Count; i++)
            {
                if (sc.Value > scores[i].Value)
                {
                    scores.Insert(i, sc);
                    added = true;
                    break;
                }
            }

            if(!added && scores.Count < 10)
            {
                scores.Add(sc);
                SaveHighScores();
            }
            else if(added && scores.Count > 10)
            {
                for (int i = 10; i < scores.Count; i++)
                {
                    scores.RemoveAt(i);
                    i--;
                }
            }
        }

        static public tScore GetScore(int pos)
        {
            if (pos < scores.Count)
                return scores[pos];
            else return new tScore();
        }

        static public void LoadHighScores()
        {
            scores = new List<tScore>();

            if(File.Exists(scoreFile))
            {
                string[] lines = File.ReadAllLines(scoreFile);

                tScore tmp;
                int value;
                foreach(string l in lines)
                {
                    for(int i = 0; i < l.Length; i++)
                    {
                        if(l[i] == '-')
                        {
                            tmp.Name = l.Substring(0, i - 1);
                            if (int.TryParse(l.Substring(i + 2), out value))
                            {
                                tmp.Value = value;
                                scores.Add(tmp);
                            }
                            break;
                        }
                    }
                }
            }
        }

        static public void SaveHighScores()
        {
            using(StreamWriter sw = File.CreateText(scoreFile))
            {
                foreach (tScore sc in scores)
                {
                    sw.WriteLine(sc.ToString());
                }
            }
        }
    }
}
