﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MathGame
{
    public enum PrmType
    {
        String,
        Int,
        Double,
        Enum,
    }

    public struct tVal
    {
        public double d;
        public int i;
        public string s;
    }

    public class UsrPrm
    {
        public string ID;
        public string hint;

        public PrmType type;

        public tVal value;

        public tVal max;
        public tVal min;
        public int prec;
        public string[] txtList;
        // Jednotky
        // 0 - mm
        // 1 - um
        public int units; 

        public UsrPrm()
        {
            ID = string.Empty;
            hint = string.Empty;

            type = PrmType.String;

            prec = 0;
            txtList = null; 
        }

        // String Prm
        public UsrPrm(PrmType type, string ID, string hint, string def)
        {
            this.ID = ID;
            this.hint = hint;
            this.type = type;
            value = new tVal();
            value.s = def;
            txtList = null;
        }

        // Integer Prm
        public UsrPrm(PrmType type, string ID, string hint, int val, int max, int min)
        {
            this.ID = ID;
            this.hint = hint;
            this.type = type;
            value = new tVal();
            value.i = val;
            this.max = new tVal();
            this.max.i = max;
            this.min = new tVal();
            this.min.i = min;
            txtList = null;
        }

        // Double Prm
        public UsrPrm(PrmType type, string ID, string hint, double val, double max, double min, int prec)
        {
            this.ID = ID;
            this.hint = hint;
            this.type = type;
            value = new tVal();
            value.d = val;
            this.max = new tVal();
            this.max.d = max;
            this.min = new tVal();
            this.min.d = min;
            this.prec = prec;
            txtList = null;
        }

        // Enum Prm
        public UsrPrm(PrmType type, string ID, string hint, int val, int max, string[] txtlist)
        {
            this.ID = ID;
            this.hint = hint;
            this.type = type;
            value = new tVal();
            value.i = val;
            this.max = new tVal();
            this.max.i = max;
            this.txtList = txtlist;
        }

        public object GetValue()
        {
            switch(type)
            {
                case PrmType.Double:
                    return value.d;

                case PrmType.Enum:
                    return txtList[value.i];

                case PrmType.Int:
                    return value.i;

                case PrmType.String:
                    return value.s;
            }
            return null;
        }

        public string GetTooltip()
        {
            if (type == PrmType.Double)
            {
                string strUnits = units == 0 ? "mm" : "um";
                return "( " + min.d.ToString("G" + prec.ToString()) + " - " + max.d.ToString("G" + prec.ToString()) + " ["+ strUnits +"] )";
            }
            else if (type == PrmType.Int)
            {
                string strUnits = units == 0 ? "mm" : "um";
                return "( " + min.i + " - " + max.i + " [" + strUnits + "] )";
            }
            // pri null hodnote se neukazuje tooltip vubec
            // pokud se posle string.Empty tak se ukaze prazdny tooltip coz neni idealni
            else return null;
        }

        public bool CheckValue(object val)
        {
            switch (type)
            {
                case PrmType.Double:
                    if (min.d <= (double)val && (double)val <= max.d)
                        return true;
                    else return false;

                case PrmType.Int:
                    if (min.i <= (int)val && (int)val <= max.i)
                        return true;
                    else return false;
            }
            return true;
        }

        public void SetValue(object val)
        {
            switch (type)
            {
                case PrmType.Double:
                    value.d = (double) val;
                    break;

                case PrmType.Enum:
                case PrmType.Int:
                    value.i = (int)val;
                    if (value.i > max.i)
                        value.i = 0;
                    break;

                case PrmType.String:
                    value.s = (string)val;
                    break;
            }
        }

        public override string ToString()
        {
            //return base.ToString();
            string strVal;
            switch(type)
            {
                case PrmType.Double:
                    strVal = value.d.ToString();
                    break;

                case PrmType.Enum:
                case PrmType.Int:
                    strVal = value.i.ToString();
                    break;

                case PrmType.String:
                    strVal = value.s;
                    break;

                default:
                    strVal = "0000";
                    break;
            }
            return ID.ToString() + " = " + strVal;
        }

        public object ParseValue(string s)
        {
            object val;
            switch (type)
            {
                case PrmType.Double:
                    try { val = double.Parse(s.Replace('.', ',')); }
                    catch { val = -5; }
                    break;

                case PrmType.Enum:
                case PrmType.Int:
                    try { val = int.Parse(s); }
                    catch { val = -5; }
                    break;

                case PrmType.String:
                    val = s;
                    break;

                default:
                    val = "unknown type";
                    break;
            }
            return val;
        }
    }
}
